/**
 * erropts.h
 * 98/11/22
 * Eric Sommerlade
 * 
 * header for error definition parsing
 *
 */
#ifndef ERROPTS_H_
#define ERROPTS_H_

#ifndef NULL
#define NULL (0)
#endif

typedef enum { False=0, True=1 } TBoolean;
typedef void *TErrInfo;
typedef TBoolean (validchk)(int num, TErrInfo *e_s);

/** functions to yield access to error structure e_s
 *  validPic(p,e_s) returns validity of picture p
 *  validSlc(s,e_s) returns validity of slice s
 *  validMba(m,e_s) returns validity of masterblock address m
 *  validBlk(b,e_s) returns validity of block b
 * 
 *  @param num: full integer range
 *  @param e_s: all valid (TErrInfo *) including NULL
 */
extern validchk validPic, validSlc, validMba, validBlk, validBit;


/** Flag to specify wether the error generator should 
 *  output error insertion messages
 * automatically set in processErrorOptions
 */
extern TBoolean Error_Quiet_Flag;

/*
 * prototypes:
 *
 */

/** Initializes the TErrInfo structure Err_Info.
 *
 * @return (TErrInfo *) if successful
 *         NULL         if file error occurred
 */
TErrInfo *processErrorOptions(char *filename);

/** Deinitializes the TErrInfo structure Err_Info.
 *  sets Err_Info to NULL, but does not free it!
 * @param e_s - the error information structure 
 *              to be freed.
 */
void freeErrorOptions(TErrInfo *e_s);


/** global variable to index current error list
 *  initialized with the last return value of 
 *  processErrorOptions
 *  deinitialized by freeErrorOptions (set to NULL)
 */
extern TErrInfo *Err_Info;

#endif





  


