
/* libmpeg.c: initialization, option processing                    */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include "config.h"
#include "global.h"
#include "mpeg2decfunctions.h"
#include "libmpeg2.h"

int initializeDecoder(int argc, char *argv[])
{
  Clear_Options();

  /* decode command line arguments */
  if(!Process_Options(argc,argv))
    return(0);

  return(1);
}



TStreamInfo *initializeSequence() 
{

  ld = &base; /* select base layer context */

  /* open MPEG base layer bitstream file(s) */
  /* NOTE: this is either a base layer stream or a spatial enhancement stream */
  if ((base.Infile=open(Main_Bitstream_Filename,O_RDONLY|O_BINARY))<0)
  {
    fprintf(stderr,"Base layer input file %s not found\n", Main_Bitstream_Filename);
    return(NULL);
  }


  if(base.Infile != 0)
  {
    int code;
    Initialize_Buffer(); 
  
    if(Show_Bits(8)==0x47)
    {
      sprintf(Error_Text,"Decoder currently does not parse transport streams\n");
      Error(Error_Text);
    }

    next_start_code();
    code = Show_Bits(32);

    switch(code)
    {
    case SEQUENCE_HEADER_CODE:
      break;
    case PACK_START_CODE:
      System_Stream_Flag = 1;
    case VIDEO_ELEMENTARY_STREAM:
      System_Stream_Flag = 1;
      break;
    default:
      sprintf(Error_Text,"Unable to recognize stream type\n");
      Error(Error_Text);
      break;
    }

    lseek(base.Infile, 0l, 0);
    Initialize_Buffer(); 
  }

  if(base.Infile!=0)
  {
    lseek(base.Infile, 0l, 0);
  }

  Initialize_Buffer(); 

  if(Two_Streams)
  {
    ld = &enhan; /* select enhancement layer context */

    if ((enhan.Infile = open(Enhancement_Layer_Bitstream_Filename,O_RDONLY|O_BINARY))<0)
    {
      sprintf(Error_Text,"enhancment layer bitstream file %s not found\n",
        Enhancement_Layer_Bitstream_Filename);
      Error(Error_Text);
      Two_Streams=0;
    }

    Initialize_Buffer();
    ld = &base;
  }

  ASSERT(Initialize_Decoder());



  {
    int ret;
    int Bitstream_Framenum;
    TStreamInfo *si;

    Bitstream_Framenum = 0;

    do {
      ret=Headers();
    } while(ret>>1); /*ret==1: image detected, ret==0: end of sequence det.*/
    
    if(ret==0) return(NULL); /* end of sequence detected */
    /* at this point the first image has been decoded partially */
    Initialize_Sequence();

    si=(TStreamInfo *)malloc(sizeof(TStreamInfo));
    si->chroma=chroma_format;
    si->codedw=Coded_Picture_Width;
    si->w=horizontal_size;
    si->h=vertical_size;
    si->aspect=aspect_ratio_information;
    si->framerate=frame_rate;
    si->bitrate=bit_rate;
    si->bsFramenum=0;
    si->sqFramenum=0;
    si->bsFieldnum=0;
    si->endReached=FALSE;


    
    
    
    return(si);
  };


  
  return(NULL);
}

void deinitializeSequence(TStreamInfo *stream) 
{

  close(base.Infile);

  if (Two_Streams)
    close(enhan.Infile);

  if(stream!=NULL)
    free(stream);

}

void deinitializeDecoder() 
{
  Clear_Options();
}



TPictureInfo *nextPicture(TStreamInfo *si) 
{
  /* decode picture whose header has already been parsed in 
     initializeSequence() */
  if(!si->bsFieldnum) {
    Decode_Picture(0, 0);
  } /*else*/

  {
    if(Headers()) {

      
      /* @TODO: check for interlacing!!! */
      
      Decode_Picture(si->bsFramenum+1, si->sqFramenum+1);

    
    } else {
     /* put last frame */
      if (si->sqFramenum!=0 && !si->endReached) {
        Output_Last_Frame_of_Sequence(si->bsFramenum);
        si->endReached=TRUE;
      } else 
        return(NULL); /* not a single frame/field found! or no more frames */
    }
  }
  /* update picture numbers */
  si->bsFieldnum++;
  if (picture_structure==FRAME_PICTURE || !Second_Field) {
    si->bsFramenum++;
    si->sqFramenum++;
  }
  {
    unsigned char **curfrm;
    TPictureInfo *pi;
    
    if (picture_coding_type==B_TYPE)
      curfrm=auxframe;
    else
      curfrm=forward_reference_frame;

    pi=&si->curFrame;
    pi->Y=curfrm[0];
    pi->Cr=curfrm[1];
    pi->Cb=curfrm[2];

    return(pi);
  }
  
}
