/***************************************************************************
                          getPicture.cpp  -  description
                             -------------------
    begin                : |DATE|
    copyright            : (C) |YEAR| by |AUTHOR|
    email                : tuvisionlib@tuvision.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
/**********************************************************/
/* Example for mpeg stream disassembly. 
 * Only the first picture is read and filtered.
 * \author Eric Sommerlade
 */
/**********************************************************/


#include <iostream>

#include "tuv/CImageByteRGB.h"
#include "tuv/CImageByteYUV.h"
#include "tuv/CException.h"
#include "tuv/CFileWriteTiff.h"
#include "tuv/CFileReadMpeg.h"

#include "tuv2d/COperatorRGB.h"


using namespace std;
using namespace TUVision;

/*!  main routine.
 *   arguments: 
 *    mpeg-filename          - the source.
 *    output-filename        - the output file name (always saved as tif)
 */
int main (int argc, char** argv)
{
    CFileReadMpeg   src;
    CFileWriteTiff  dst;
    CImageByteYUV       *img;

    if(argc<3) {
        cerr << "usage: " << *argv << " infile(mpeg format) outfile-prefix(.tif)"
            << endl
            << "       The tool reads the first picture from an mpeg "
            << "stream."
            << endl << endl;
        return(-1);
    }

    try {

        cout << "Opening file " << argv[1] << endl;
        // the mpeg file source is opened here.
        src.open (argv[1]);

        // now convert to rgb-image for saving.
        img = dynamic_cast<CImageByteYUV*> (src.read());
        CImageByteRGB rgb(convertToRGB(*img));
        // delete old yuv image.
        delete img;

        // open output file
        dst.open (argv[2]);
        cout << "Writing: " << argv[2] << endl; 

        // write the rgb picture as a tiff
        dst.write (rgb);
        // close the file.
        dst.close();

        // close the mpeg file source
        src.close();
    }
    catch (CException &e) { 
        // treat the library errors.
        cerr << "Error: " << e.what() << endl; 
    }
    catch (std::exception e) {
        // treat std c++ errors.
        cerr << "std exception caught: " << e.what() << endl; 
    } 
    catch (...) {
        // treat unknown errors.
        cerr << "Unknown exception caught." << endl; 
    } 
    return (0);
}
