/***************************************************************************
                          CFileRead.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREAD_H_
#define __TUVISION_CFILEREAD_H_ 1

#include "TUVisionDef.h"
#include "tuv/CImage.h"
#include "tuv/CFile.h"


namespace TUVision {

/***********************************************************/
//!  Class to read graphic files.
/*! 
 *   This class offers an API to read graphic files in
 *   the memory. Graphic formats with more than one 
 *   image per file are supported. So it is possible to
 *   use this interface for video files, too.
 *
 *   If necessary derived classes should offer additional
 *   "set" functions. For example these functions could be 
 *   used to enable/disable compression.
 *
 *   \ingroup io
 * 
 *   \author Marco Budde 
 */
/***********************************************************/

class _CCTUVISION CFileRead : public CFile {

public:
    /*******************************************************/
    //!  Read image to memory.
    /*! 
     *   Read image to memory. As the format (color model 
     *   etc.) of the image is unknown during the compilation
     *   of the program, this function uses CImage as
     *   return type. So it is necessary to convert the
     *   return type using RTTI (typeid, dynamic_cast).
     *
     *   \param img_number read image number "img_number"
     *   \return the image
     */ 
    /*******************************************************/
    virtual CImage *read (int img_number = 0) = 0;

    /*******************************************************/
    //!  Get number of images.
    /*! 
     *   How many images are stored in the file? In cases
     *   where it is not possible to calculate the number
     *   of images, this function will return "-1".
     *
     *   \return number of images
     */
    /*******************************************************/
    virtual int getNumberImages (void) = 0;
};

}

#endif  // __TUVISION_CFILEREAD_H_
