/***************************************************************************
                          CFileReadAll.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADALL_H__
#define __TUVISION_CFILEREADALL_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"
#include "tuv/CFileRead.h"
#include "tuv/CFileReadTiff.h"
#include "tuv/CFileReadJpeg.h"
#include "tuv/CFileReadBmp.h"
#include "tuv/CFileReadPng.h"
#include "tuv/CFileReadMpeg.h"

namespace TUVision {

//! CFileRead factory.
/*! \class CFileReadAll CFileReadAll.h tuv/CFileReadAll.h
 *  This class serves as a factory for
 *  CFileRead objects.
 *  You specify the filename, the file gets opened
 *  and the first few bytes are read.
 *  If the bytes read can be found in an internal table,
 *  the corresponding non abstract CFileRead daughter 
 *  class is instantiated.
 *
 *  \warning
 *  If the bytes read do not correspond to any file type,
 *  an CExceptionFile is thrown.
 *
 *  To install a new file type to this class,
 *  refer to the documentation of CFileReadRegistry,
 *  the class that keeps all information on file read
 *  classes.
 *
 *  \author Eric Sommerlade
 *
 *  \class CFileReadAll CFileReadAll.h tuv/CFileReadAll.h
 */
class _CCTUVISION CFileReadAll: public CFileRead {
public:
    CFileReadAll();
    ~CFileReadAll();

    void open(const std::string& filename);
    void close();
    CImage* read(int img_number = 0);
    int getNumberImages();
private:
    CFileRead* d_fileread;
    void InitializeDefaultRegistrators();
};

}
#endif // __TUVISION_CFILEREADALL_H__
