/***************************************************************************
                          CFileReadRegistrator.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __CFILEREADREGISTRATOR_H__
#define __CFILEREADREGISTRATOR_H__ 1

#include "TUVisionDef.h"

namespace TUVision {

class CFileRead;

//! Encapsulator of the registration and finding procedure.
/*! 
 * Encapsulates the registration procedure and
 * the check if two arrays start with the same bytes.
 *
 * For more complete information, refer to the documentation
 * of CFileReadRegistry.
 *
 * \author Eric Sommerlade
 *
 * \class CFileReadRegistrator CFileReadRegistrator.h tuv/CFileReadRegistrator.h
 */
class _CCTUVISION CFileReadRegistrator {
    //! the length of the id byte array.
    const int d_len;
    //! the byte array identifying the file to read.
    const char* d_id;
public:
    //! abstract method to construct file reader.
    virtual TUVision::CFileRead* construct()=0;
    //! method to check if the passed in arry starts with the same bytes.
    virtual bool startsWith(int len, const unsigned char* id);
    //! Gets the length of the magic string
    virtual const int getLength()=0;
    //! Gets the magic string
    virtual const unsigned char *getID()=0;
    //! registers the registrator in the registry.
    CFileReadRegistrator();
    //! removes the registrator from the registry.
    virtual ~CFileReadRegistrator();
};

} // namespace TUVision

#endif // __CFILEREADREGISTRATOR_H__
