/***************************************************************************
                          CFileReadTiff.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADTIFF_H_
#define __TUVISION_CFILEREADTIFF_H_ 1

#include <exception>
#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include "tuv/CFileReadRegistrator.h"

typedef	struct tiff TIFF;

namespace TUVision {

/**********************************************************/
//!  Reads a TIFF graphic file.
/*! 
 *   This class reads a TIFF graphic file to memory.
 *   It is possible to read several images from one
 *   TIFF graphic file. Simply call the read() function
 *   several times.
 *
 *   The returned image objects could be of the
 *   following types: CImageByteBilevel, CImageByteGray,
 *   or CImageByteRGB. 
 *
 *   The following CImageAttributes are supported:
 *   xdpi, and ydpi.
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 *
 *   \class CFileReadTiff CFileReadTiff.h tuv/CFileReadTiff.h
 */
/**********************************************************/

class _CCTUVISION CFileReadTiff : public CFileRead {

public:
    CFileReadTiff (void);
    ~CFileReadTiff();

    void open (const std::string &filename) 
        throw (CException);

    void close (void) 
        throw();

    CImage *read (int img_number = 0) 
        throw (std::exception, CException);

    int getNumberImages (void) 
        throw (CException);

private:
    /******************************************************/
    //!  Read the attributes of a TIFF file.
    /******************************************************/
    void readAttributes (CImage &img);

private:
    TIFF  *d_tif_hd;   //!< handle of libtiff
};

// static class to register the file reader
class CFileReadTiffBigEndianRegistrator: public CFileReadRegistrator {
    static unsigned const char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadTiff());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};


// static class to register the file reader
/*! \class CFileReadTiffLittleEndianRegistrator CFileReadTiff.h tuv/CFileReadTiff.h */
class CFileReadTiffLittleEndianRegistrator: public CFileReadRegistrator {
    static unsigned const char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadTiff());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};

} 

#endif // __TUVISION_CFILEREADTIFF_H
