/***************************************************************************
                          CFileWrite.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITE_H_
#define __TUVISION_CFILEWRITE_H_ 1


#include "TUVisionDef.h"
#include "tuv/CFile.h"


namespace TUVision {

/***********************************************************/
//!  Write images to a graphic file.
/*! 
 *   Class to write images from memory to the disc.
 *   Derived classes have to define at least one
 *   write() function. Normally a derived class will
 *   have several write() functions. For example:
 *   "write (CImageByteGray &)" and 
 *   "write (CImageByteRGB &)". 
 *
 *   File formats that can store more than one image are 
 *   supported. The user can call write() several times 
 *   to store several images in one file (if supported 
 *   by the file format).
 *
 *   If necessary derived classes should offer a 
 *   function setParameters().
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 */
/***********************************************************/

class _CCTUVISION CFileWrite : public CFile {

public:
    virtual void open (const std::string &filename)
        throw (CException)=0;

    virtual void close (void)
        throw ()=0;

    virtual void write (const CImageByteGray &img)
        throw (CException)=0;

    virtual void write (const CImageByteRGB &img)
        throw (CException)=0;

};

}

#endif // __TUVISION_CIMAGEFILEWRITE_H_
