/***************************************************************************
                          CFileWritePbm.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITEPBM_H_
#define __TUVISION_CFILEWRITEPBM_H_ 1

#include <fstream>
#include "tuv/CImageDatatypes.h"

#include "tuv/CFileWrite.h"
#include "tuv/CException.h"


namespace TUVision {


/**********************************************************/
/*! 
 *   Saves an image as PBM/PGM/PPM graphic file. This
 *   file format is supported by most UNIX/Linux image
 *   processing programs. The main disadvantage of this
 *   file format is the size of the files, because they
 *   are stored as an ascii format.
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFileWritePbm: public CFileWrite {

public:
    CFileWritePbm (void);
    ~CFileWritePbm();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw ();

    /******************************************************/
    //!  Save 8 bit gray image.
    /******************************************************/
    void write (const CImageByteGray &img)
        throw (CException);

    /******************************************************/
    //!  Save 8 bit RGB image.
    /******************************************************/
    void write (const CImageByteRGB &img)
        throw (CException);

    /******************************************************/
    //!  Save bilevel image.
    /******************************************************/
    void write (const CImageByteBilevel &img)
        throw (CException);


private:
    std::ofstream  hd;       //!< file i/o class
};

}

#endif  // __TUVISION_CFILEWRITEPBM_H_
