/***************************************************************************
                          CFileWritePng.h  -  description
                             -------------------
    begin                : 20.09.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Preparation Bildverarbeitung 1 WS 2001

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITEPNG_H_
#define __TUVISION_CFILEWRITEPNG_H_ 1

#include <cstdio>
#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"
#include "tuv/CFileWrite.h"
#include "tuv/CException.h"



namespace TUVision {

/**********************************************************/
//!  Saves an image as PNG graphic file.
/*! 
 *   Saves an image as PNG graphic file.
 *   PNG is used as as a graphic file format for
 *   lossless compression. It works more effiently than GIF
 *   for example by also providing many features like
 *   transparency. Its power lies in runlength compression
 *   of color-indexed files.
 *
 *   The returned image objects could be of type 
 *   CImageByteGray or CImageByteRGB. A JPEG file could
 *   contain only one image.
 *
 *   \ingroup io
 *
 *   \author Fabian Wenzel
 */
/**********************************************************/

class _CCTUVISION CFileWritePng: public CFileWrite {

public:
    CFileWritePng (void);
    ~CFileWritePng();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw();

    /******************************************************/
    //! Save 8 bit gray image.
    /******************************************************/
    void write (const CImageByteGray &img)
        throw (CException);

    /******************************************************/
    //! Save 8 bit RGB image.
    /******************************************************/
    void write (const CImageByteRGB &img)
        throw (CException);

private:
    /******************************************************/
    //! Save 8 bit image.
    /******************************************************/
    void write (const CImageByte &img, int color_space)
        throw (CException);

private:
    FILE  *d_hd;          //!< file handle
};

}

#endif
