/***************************************************************************
                           CPlane.h  -  description
                              -------------------
     begin                : 23.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CPLANE_H__
#define __TUVISION_CPLANE_H__	1

#include <tuv/CVertex.h>

#include <TUVisionDef.h>

namespace TUVision
{
/**********************************************************/
//!  This class represents a plane.
/*! 
 *   The plane is represented in 3d space by an orthonormal
 *   vector n of euclidean length 1 and the euclidean distance d
 *   to the coordinate system origin.
 *
 *   Coordinates x = (x1,x2,x3)^T lie in the plane if following
 *   equation is true: x n = d.
 *
 *   \ingroup geometry
 *
 *   \author Marc Hensel
 *
 *   \class CPlane CPlane.h tuv/CPlane.h
 */
/**********************************************************/

class _CCTUVISION CPlane
{
private:

public:
    double	d;				//!< Distance to origin of coordinate system
    double	nx;				//!< x-component of orthonormal vector n with ||n|| = 1
    double	ny;				//!< y-component of orthonormal vector n with ||n|| = 1
    double	nz;				//!< z-component of orthonormal vector n with ||n|| = 1

    //! Constructor setting plane by distance and orthonormal vector
    /*!
     *  \param	normalize	Vector n is not normalized to length 1 if set to "false"
     */
    CPlane	(double d,double nx,double ny,double nz,bool normalize=true);

    CPlane	();											//!< Constructor setting d=0, nx=ny=0, and nz=1
    CPlane	(CVertex3d P1,CVertex3d P2,CVertex3d P3);	//!< Constructor setting plane using three given points in plane
    ~CPlane	();											//!< Destructor

    //! Set plane using three given points in plane
    bool	set3PointPlane	(CVertex3d P1,CVertex3d P2,CVertex3d P3);

    bool	isInPlane		(CVertex3d x);					//!< Returns "true" if x lies in plane
    bool	isInPlane		(double x,double y,double z);	//!< Returns "true" if (x,y,z)^T lies in plane

    //! Normalize n to euclidean length = 1 after changing nx, ny, and/or nz.
    /*!
     *  \return	 Returns "false" if vector is nullvector nx = ny = nz = 0.
     */
    bool	normVector		();
};
}

#endif	// __TUVISION_CPLANE_H__
