/***************************************************************************
                          CPoint.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CPOINT_H__
#define __TUVISION_CPOINT_H__ 1

#include "TUVisionDef.h"

namespace TUVision {


/**********************************************************/
//!  Class defining a point.
/*!
 *   This class defines a point with x- and y-coordinates.
 *
 *   \ingroup geometry
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CPoint
{
public:

    /******************************************************/
    //! Default constructor.
    /******************************************************/
    CPoint();

    /******************************************************/
    //! Creates a point with the given coordinates.
    /******************************************************/
    CPoint (int x, int y);

    /******************************************************/
    //! Copy constructor.
    /******************************************************/
    CPoint (const CPoint &pnt);

    /******************************************************/
    //! Sets both coordinates of the point.
    /******************************************************/
    void setPoint (int x, int y);

    /******************************************************/
    //! Sets the x-coordinates of the point.
    /******************************************************/
    void setX (int x);

    /******************************************************/
    //! Sets the y-coordinates of the point.
    /******************************************************/
    void setY (int y);

    /******************************************************/
    //! Reads the x-coordinates of the point.
    /******************************************************/
    inline int getX (void) const;

    /******************************************************/
    //! Reads the y-coordinates of the point.
    /******************************************************/
    inline int getY (void) const;

    /******************************************************/
    //! Copies the coordinates of an existing point.
    /******************************************************/
    CPoint &operator= (const CPoint &rhs);

    /******************************************************/
    //! Checks for point equality..
    /******************************************************/
    bool operator== (const CPoint &pt) const;

    /******************************************************/
    //! Checks for point inequality..
    /******************************************************/
    bool operator!= (const CPoint &pt) const;

    /**********************************************************/
    //! subtracts two points.
    /**********************************************************/
    CPoint& operator-=(const CPoint& p);
    /**********************************************************/
    //! adds two points.
    /**********************************************************/
    CPoint& operator+=(const CPoint& p);

    /**********************************************************/
    //! returns the square of the euclidian norm.
    /**********************************************************/
    int getLength() const;


private:    
    int d_x;  //!< x-coordinate
    int d_y;  //!< y-coordinate
};


/**********************************************************/
/*! 
 *   Reads the x-coordinate of the point.
 *
 *   \return x-coordinate
 */
/**********************************************************/

int CPoint::getX (void) const
{
    return (d_x); 
}


/**********************************************************/
/*! 
 *   Reads the y-coordinate of the point.
 *
 *   \return y-coordinate
 */
/**********************************************************/

int CPoint::getY (void) const
{
    return (d_y); 
}

}

#endif
