/***************************************************************************
                           CFilterContrast.h  -  description
                              -------------------
     begin                : 30.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CFILTERCONTRAST_H__
#define __TUVISION_CFILTERCONTRAST_H__	1

#include <tuv/CImageByteGray.h>

#include <TUVisionDef.h>

namespace TUVision 
{

/**********************************************************/
//!  Contrast filter.
/*! 
 *   Each filtered pixel intensity is replaced by the contrast
 *   in a nxn-pixel-neighborhood.
 *
 *   Contrast is defined as c = max_intensity - min_intensity.
 *
 *   \ingroup filter
 *
 *   \author Marc Hensel
 *
 *   \class CFilterContrast CFilterContrast.h tuv2d/CFilterContrast.h
 */
/**********************************************************/

class _CCTUVISION CFilterContrast
{
private:

public:
    //! Constructor
    CFilterContrast		();

    //! Destructor
    ~CFilterContrast	();

    //! Filter, i.e. replace pixel intensity with local contrast (max_value - min_value) in n x n window.
    /*!
     *  \param	img	Image to be filtered (overwritten with results)
     *  \param	filterWidth	Width n of filter window (must be odd)
     */
    bool	filter		(CImageByteGray *img,unsigned char filterWidth);
};

}

#endif	// __TUVISION_CFILTERCONTRAST_H__

