/***************************************************************************
                           CFilterMedian.h  -  description
                              -------------------
     last modified        : 30.4.2004 Dennis Voss
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Study thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

/*
 * Note:
 *
 * Method 'median' uses sort() from STL <algorithm>
 */

#ifndef __TUVISION_CFILTERMEDIAN_H__
#define __TUVISION_CFILTERMEDIAN_H__	1

#include <tuv/CImageByteGray.h>

#include <TUVisionDef.h>

namespace TUVision 
{

/**********************************************************/
//!  Median filter (noise reduction).
/*! 
 *   This class implements the Median filter used for
 *   noise reduction. Filter width is odd per definition
 *   and remains unchanged when trying to set to an even
 *   value.
 *
 *   \ingroup filter
 *
 *   \author Marc Hensel
 *
 *   \class CFilterMedian CFilterMedian.h tuv2d/CFilterMedian.h
 */
/**********************************************************/

class _CCTUVISION CFilterMedian
{
private:
    unsigned char	filterWidth;						//!< Filter width (must be odd value)

public:
    CFilterMedian	(unsigned char width=3);			//!< Constructor (sets filter width to 3 if called without argument)
    ~CFilterMedian	();									//!< Destructor

    bool			setWidth	(unsigned char width);	//!< Set filter width (must be an odd value)
    unsigned char	getWidth	();						//!< Return current filter width

    //! Filter a 1-dimensional array.
    /*!
     *  \param	data_array	Pointer to data to be filtered
     *  \param	array_length	Size of data_array in number of entries
     */
    bool			filter		(int *data_array,const int array_length);
    bool			filter		(unsigned char *data_array,const int array_length);
    bool			filter		(double *data_array,const int array_length);
    bool			filter		(float *data_array,const int array_length);

    //! Filter an image (2d filter).
    /*!
     *  \param	img	Image to be filtered
     */
    bool			filter		(CImageByteGray &img);
};

}

#endif	// __TUVISION_CFILTERMEDIAN_H__
