/***************************************************************************
                           CFilterSumInversion.h  -  description
                              -------------------
     begin                : 22.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Study thesis Marc Hensel
 
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CFILTERSUMINVERSION_H__
#define __TUVISION_CFILTERSUMINVERSION_H__	1

#include <tuv2d/CFilterByteGray.h>

#include <TUVisionDef.h>

namespace TUVision 
{

/**********************************************************/
//!  Sum inversion filter.
/*! 
 *   This class sums up all pixels in the filter window
 *   and normalizes so that the lowest sum is mapped to 255
 *   and the highest sum is mapped to zero. Filter width is
 *   odd per definition and remains unchanged when trying to
 *   set to an even value.
 *
 *   \ingroup filter
 *
 *   \author Marc Hensel
 *
 *   \class CFilterSumInversion CFilterSumInversion.h tuv2d/CFilterSumInversion.h
 */
/**********************************************************/

class _CCTUVISION CFilterSumInversion : public CFilterByteGray
{
private:
    unsigned char	filterWidth;	//!< Filter width (must be odd value)

public:
    //! Constructor (sets filter width to 5 if called without argument)
    CFilterSumInversion		(unsigned char width=5);

    //! Destructor
    ~CFilterSumInversion	();

    //! Set filter width (must be odd value)
    bool			setWidth	(unsigned char width);

    //! Return current filter width
    unsigned char	getWidth	();

    //! Filter the image
    void			filter		(CImageByteGray &img);

    //! Set areas with value >= threshold at border of filtered image to zero.
    /*!
     *  The border of the filtered image is not the border of the original
     *  image, i.e. the outer lines creating a filtered value.
     *  E.g.: y=0 is no border but leads to y=(int)(filter_width/2).
     */
    void			removeBorder(CImageByteGray &img,unsigned char threshold=255);
};

}

#endif	// __TUVISION_CFILTERSUMINVERSION_H__
