/***************************************************************************
                          CFilterSymmetricCovariance.h  -  description
                             -------------------
    begin                : 06.04.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library's internal version. It is   *
 *   not permitted to publish this file at this point of time as the       *
 *   algorithms are meant to be shared between TUHH Vision Systems         *
 *   members only.                                                         *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILTERSYMMETRICCOVARIANCE_H__
#define __TUVISION_CFILTERSYMMETRICCOVARIANCE_H__ 1

#include "tuv2d/CFilterByteGray.h"
#include "tuv2d/CFilterIntGray.h"
#include "tuv2d/CFilterDoubleGray.h"

#include "TUVisionDef.h"


namespace TUVision 
{

/**********************************************************/
//!  Symmetric Covariance filter (symmetric point detection)
/*! 
 *   \class CFilterSymmetricCovariance CFilterSymmetricCovariance.h tuv2d/CFilterSymmetricCovariance.h 
 *   This class implements the symmetric covariance filter
 *   which can be used for symmetric point detection. It
 *   is especially used when looking for corner points
 *   in a checkerboard pattern as part of a calibration object.
 *   An evaluation can be found e.g.
 *   in Fabian Wenzel's Diploma Thesis. The filterwidth which gets
 *   set in CFilterSymmetricCovariance::init() has to be odd!
 *
 *   \ingroup filter
 *
 *   \author Fabian Wenzel
 */
/**********************************************************/

class _CCTUVISION CFilterSymmetricCovariance : public CFilterByteGray{
public:

    //! Initializes the filter with the default filter width 13.
    CFilterSymmetricCovariance();

    //! Filters the image using the set filterwidth
    void filter (CImageByteGray &img);

    //! Sets the filter width in pixels. It has to be odd!
    void init(int filterwidth);
private:
    int d_filterwidth;
};


} 

#endif // __TUVISION_CFILTERSYMMETRICCOVARIANCE_H__
