/***************************************************************************
                           COperatorContrast.h  -  description
                              -------------------
     begin                : 30.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_COPERATORCONTRAST_H__
#define __TUVISION_COPERATORCONTRAST_H__	1

#include <tuv/CImageByteGray.h>

#include <TUVisionDef.h>

namespace TUVision 
{

/**********************************************************/
//!  Contrast operations.
/*! 
 *   Contrast is defined as c = max_intensity - min_intensity.
 *
 *   \ingroup operator
 *
 *   \author Marc Hensel
 *
 *   \class COperatorContrast COperatorContrast.h tuv2d/COperatorContrast.h
 */
/**********************************************************/

class _CCTUVISION COperatorContrast
{
private:

public:
    //! Constructor
    COperatorContrast	();

    //! Destructor
    ~COperatorContrast	();

    //! Linear transform so that the image intensities are stretched to the full dynamic range [0,255].
    /*!
     *  The min. and max. intensities i_min and i_max in the image are determined
     *  and mapped linear i_min -> 0 and i_max -> 255 so that the full dynamic range
     *  is used.
     */
    void	maxContrast			(CImageByteGray *img);

    //! Logarithmic mapping of the dynamic range to itself [0,255]->[0,255].
    /*!
     *  At first the image intensities are stretched linear to the full
     *  dynamic range according of the method description for maxContrst().
     *
     *  Afterwards for each pixel intensity i the function \f$ f(i) = 46 \cdot ln[i + 1] \f$
     *  is applied resulting in an logarithmic amplification towards higher intensities.
     *
     *  The parameters of f:
     *
     *  - Source data is of range [0,255], where log(0) is undefined. Therefore "1" is 
     *    added to each pixel intensity before applying the logarithm.
     *  - The factor 46 and taking the integer value leads to the desired
     *    dynamic range of [0,255] as f(0)=0 and (int)(f(255))=255.
     */
    void	maxLogContrast		(CImageByteGray *img);
};

}

#endif	// __TUVISION_COPERATORCONTRAST_H__

