/***************************************************************************
                          COperatorConverter.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORCONVERTER_H__
#define __TUVISION_COPERATORCONVERTER_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"


namespace TUVision {

/**********************************************************/
//!  Class to convert images between different number 
//!  formats.
/*! 
 *   This class could be used to convert a CImage from
 *   one number format to another. For example you can
 *   convert a byte gray image to a float gray image.
 *   For complex images you can also choose the type
 *   of the result (imaginary part, real part, magnitude...)
 *
 *   You cannot use this class to convert the color space
 *   of an image!
 *
 *   \ingroup operator
 *
 *   \author Marco Budde
 *   \author Eric Sommerlade
 *   \author Islam Shdaifat
 *   \author Fabian Wenzel
 */
/**********************************************************/

class _CCTUVISION COperatorConverter {

public:

    /******************************************************/
    //! Converts CImageByteGray to CImageIntGray.
    /******************************************************/
    static void convert (const CImageByteGray &imgsrc, 
        CImageIntGray **imgdst);

    /******************************************************/
    //! Converts CImageByteGray to CImageDoubleGray.
    /******************************************************/
    static void convert (const CImageByteGray &imgsrc, 
        CImageDoubleGray **imgdst);

    /******************************************************/
    //! Converts CImageIntGray to CImageByteGray.
    /******************************************************/
    static void convert (const CImageIntGray &imgsrc, 
        CImageByteGray **imgdst);

    /******************************************************/
    //! Converts CImageIntGray to CImageDoubleGray.
    /******************************************************/
    static void convert (const CImageIntGray &imgsrc, 
        CImageDoubleGray **imgdst);

    /******************************************************/
    //! Converts CImageDoubleGray to CImageByteGray.
    /******************************************************/
    static void convert (const CImageDoubleGray &imgsrc, 
        CImageByteGray **imgdst);

    /******************************************************/
    //! Converts CImageDoubleGray to CImageIntGray.
    /******************************************************/
    static void convert (const CImageDoubleGray &imgsrc, 
        CImageIntGray **imgdst);



    /******************************************************/
    //! Converts CImageByteRGB to CImageIntRGB.
    /******************************************************/
    static void convert (const CImageByteRGB &imgsrc, 
        CImageIntRGB **imgdst);

    /******************************************************/
    //! Converts CImageByteRGB to CImageDoubleRGB.
    /******************************************************/
    static void convert (const CImageByteRGB &imgsrc, 
        CImageDoubleRGB **imgdst);

    /******************************************************/
    //! Converts CImageIntRGB to CImageByteRGB.
    /******************************************************/
    static void convert (const CImageIntRGB &imgsrc, 
        CImageByteRGB **imgdst);

    /******************************************************/
    //! Converts CImageIntRGB to CImageDoubleRGB.
    /******************************************************/
    static void convert (const CImageIntRGB &imgsrc, 
        CImageDoubleRGB **imgdst);

    /******************************************************/
    //! Converts CImageDoubleRGB to CImageByteRGB.
    /******************************************************/
    static void convert (const CImageDoubleRGB &imgsrc, 
        CImageByteRGB **imgdst);

    /******************************************************/
    //! Converts CImageDoubleRGB to CImageIntRGB.
    /******************************************************/
    static void convert (const CImageDoubleRGB &imgsrc, 
        CImageIntRGB **imgdst);


    /******************************************************/
    //! Converts CImageByteYUV to CImageIntYUV.
    /******************************************************/
    static void convert (const CImageByteYUV &imgsrc, 
        CImageIntYUV **imgdst);

    /******************************************************/
    //! Converts CImageByteYUV to CImageDoubleYUV.
    /******************************************************/
    static void convert (const CImageByteYUV &imgsrc, 
        CImageDoubleYUV **imgdst);

    /******************************************************/
    //! Converts CImageIntYUV to CImageByteYUV.
    /******************************************************/
    static void convert (const CImageIntYUV &imgsrc, 
        CImageByteYUV **imgdst);

    /******************************************************/
    //! Converts CImageIntYUV to CImageDoubleYUV.
    /******************************************************/
    static void convert (const CImageIntYUV &imgsrc, 
        CImageDoubleYUV **imgdst);

    /******************************************************/
    //! Converts CImageDoubleYUV to CImageByteYUV.
    /******************************************************/
    static void convert (const CImageDoubleYUV &imgsrc, 
        CImageByteYUV **imgdst);

    /******************************************************/
    //! Converts CImageDoubleYUV to CImageIntYUV.
    /******************************************************/
    static void convert (const CImageDoubleYUV &imgsrc, 
        CImageIntYUV **imgdst);

    /******************************************************/
    //! Converts CImageByteRGB to CImageDoubleYUV.
    /******************************************************/
    static void convert (const CImageByteRGB &imgsrc, 
        CImageDoubleYUV **imgdst);

    /******************************************************/
    //! Converts CImageDoubleYUV to CImageByteRGB.
    /******************************************************/
    static void convert (const CImageDoubleYUV &imgsrc, 
        CImageByteRGB **imgdst);

    /******************************************************/
    //! Converts CImageByteRGB to CImageDoubleYUV.
    /******************************************************/
    static void convertRGBtoYUV (const CImageByteRGB &imgsrc, 
        CImageDoubleYUV **imgdst);

    /******************************************************/
    //! Converts CImageDoubleYUV to CImageByteRGB.
    /******************************************************/
    static void convertYUVtoRGB (const CImageDoubleYUV &imgsrc, 
        CImageByteRGB **imgdst);


    /******************************************************/
    //! Converts RGB tuple to YUV tuple
    /******************************************************/
    static void RGBtoLUV(const unsigned char *rgbVal, double *luvVal);

    /******************************************************/
    //! Converts YUV tuple to RGB tuple
    /******************************************************/
    static void LUVtoRGB(const double *luvVal, unsigned char *rgbVal);
private:
    static void convertByteToInt (const CImageByte &imgsrc, 
        CImageInt &imgdst,
        int nchannels);
    static void convertByteToDouble (const CImageByte &imgsrc, 
        CImageDouble &imgdst, 
        int nchannels);
    static void convertIntToByte (const CImageInt &imgsrc, 
        CImageByte &imgdst,
        int nchannels);
    static void convertIntToDouble (const CImageInt &imgsrc, 
        CImageDouble &imgdst,
        int nchannels);
    static void convertDoubleToByte (const CImageDouble &imgsrc, 
        CImageByte &imgdst,
        int nchannels);
    static void convertDoubleToInt (const CImageDouble &imgsrc, 
        CImageInt &imgdst,
        int nchannels);
};

} // namespace TUVision

#endif //__TUVISION_COPERATORCONVERTER_H__
