/***************************************************************************
                          CShape.h  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
//CShape.h
#ifndef __TUVISIONCSHAPE_H_
#define __TUVISIONCSHAPE_H_ 1
 
#include <cmath>
#include <vector>
#include <iostream>
#include "tuv2d/CPointXY.h"
#include "tuv2d/CCurve.h"

#include "TUVisionDef.h"


/********************************************************/
/*!
 * A template function rounds integer of x
 * \return round integer of x
 */
/********************************************************/
template <class T> 
int inline rint(T& x)
{	
return( int(floor(x+0.5)));
}

namespace TUVision {

/**********************************************************/
//!  Base class defines vector of CCurve
/*! 
 *   This base class defines a set or vector of curves.
 *   Each curve contains a aset of points in x,y coordinate.
 *   From this class you have to derive classes for the
 *   different shape formats like circle line etc
 *
 *   \ingroup shape
 *
 *   \author Islam Shdaifat
 */
/**********************************************************/
class _CCTUVISION CShape {

public:

    virtual ~CShape();
    CShape();

    /**********************************************************/
    /*!
     * different parameters of the curve
     * These parameters could be for example
     * lines: the angle resoultion the window size etc...
     * start point end point etc...
     * these parameters can be used later
     */
    /**********************************************************/
    std::vector <double> vecParameters;

    /**********************************************************/			
    /*!
     * vector of the shape frame points
     * the frame of the shape is the circumstance of the search
     * window
     * for arbitrary shape this can be rectangle circle ellips
     * etc....
     */
    /**********************************************************/
    std::vector < CPointXY > vecShapeFrame;

    /**********************************************************/
    /*!
     * a vector of all curves
     */
    /**********************************************************/
    std::vector < CCurve > vecCurvesLUT;

    //!get rid of the duplicated curves in vecCurvesLUT
    void getRidOfDuplicatedCurves();

    //! set the frame of the shape
    void setShapeFrame();

    void fillInLinePoints(); //! from vector in pointers
    int **LinePointsX; //! for fast search we use pointers insted of vectors
    int **LinePointsY; //! for fast search we use pointers insted of vectors
    int **LinePointsXY; //! for fast search we use pointers insted of vectors

    int nofLines; //! number of lines
    int *length;//! the length of each line

    // the image attributes
    int w_im; //<! image width
    int h_im; //<! image hight
    int ImageSize; //<! the image size w*h
    int dp; //<! the difference pointer for the image 
    unsigned char *d_image_ptr; // pointer to image date used in the loop



protected:
    CPointXY d_Pxy; //! internal use
    CCurve d_Cxy;   //! internal use

};


}; //namespace TUVision


/**********************************************************/
/*!
 * for debug purose prints out the Shape. Prints out the LUT contents
 */
/**********************************************************/
_CCTUVISION std::ostream & operator << (std::ostream& , const TUVision::CShape& );

#endif //__TUVISIONCSHAPE_H_
