/***************************************************************************
                          OpticalFlow.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_OPTICALFLOW_H__
#define __TUVISION_OPTICALFLOW_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {

/*! This file groups all optical flow calculators for gray images.
 *
 *  The optical flow of two images is calculated as follows:
 *  \f[ o_x= \frac{\delta I}{\delta t} \cdot \frac{\delta I}{\delta x} /  
 *           (norm( \nabla I ))^2 \f]
 *  \f[ o_y= \frac{\delta I}{\delta t} \cdot \frac{\delta I}{\delta y} / 
 *           (norm( \nabla I ))^2 \f]
 *
 *  \bug We have to discuss where to place such files.
 *
 *  \ingroup operator
 *
 *  \author Eric Sommerlade
 */

//! calculates the unnormalized optical flow. 
void _CCTUVISION
    calcOpticalFlowUnnormalized(const CImageByteGray& orig, 
    const CImageByteGray& next, 
    CImageIntGray** o_fx, 
    CImageIntGray** o_fy);

//! calculates the optical flow.
void _CCTUVISION
    calcOpticalFlow(const CImageByteGray& orig, 
    const CImageByteGray& next, 
    CImageDoubleGray** o_fx, 
    CImageDoubleGray** o_fy);  

//! returns a viewable version of two optical flow (x,y) images.
CImageByteRGB _CCTUVISION
    beautifyOpticalFlow(const CImageIntGray& o_fx, 
    const CImageIntGray& o_fy);

} // namespace TUVision


#endif // __TUVISION_OPTICALFLOW_H__
