/***************************************************************************
                          TsaiTransform.h  -  description
                             -------------------
    begin                : 26.03.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_TSAITRANSFORM_H__
#define __TUVISION_TSAITRANSFORM_H__ 1

#include "tuv3d/TsaiCameraModel.h"
#include "tuv/CVertex.h"
#include "TUVisionDef.h"

namespace TUVision{

/*******************************************************************************
//! This class performs coordinate transforms based on Tsai's camera model
*
* This is an adjusted version or Reg Willson's calibration code
*
* \ingroup 3d
*
\*******************************************************************************/
class _CCTUVISION CTsaiCoordTransform {
public:
    CTsaiCoordTransform();
    CTsaiCoordTransform(SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    ~CTsaiCoordTransform();
    void setCameraParameters(SCameraParameters &cameraParameters);
    void setCalibrationConstants(SCalibrationConstants &calibrationConstants);
    void Init(SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    bool undistorted2distortedSensor(TUVision::CVertex2d &undistortedCoords, TUVision::CVertex2d *distortedCoords);
    bool distorted2undistortedSensor(TUVision::CVertex2d &distortedCoords, TUVision::CVertex2d *undistortedCoords);
    bool undistorted2distortedImage(TUVision::CVertex2d &undistortedCoords, TUVision::CVertex2d *distortedCoords);
    bool distorted2undistortedImage(TUVision::CVertex2d &distortedCoords, TUVision::CVertex2d *undistortedCoords);
    bool world2image(TUVision::CVertex3d &worldCoords, TUVision::CVertex2d *imageCoords);
    bool image2world(TUVision::CVertex2d &imageCoords, double worldZValue, TUVision::CVertex3d *worldCoords);
    bool world2camera(TUVision::CVertex3d &worldCoords, TUVision::CVertex3d *cameraCoords);
    bool camera2world(TUVision::CVertex3d &cameraCoords, TUVision::CVertex3d *worldCoords);
    static bool undistorted2distortedSensor(TUVision::CVertex2d &undistortedCoords, TUVision::CVertex2d *distortedCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool distorted2undistortedSensor(TUVision::CVertex2d &distortedCoords, TUVision::CVertex2d *undistortedCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool undistorted2distortedImage(TUVision::CVertex2d &undistortedCoords, TUVision::CVertex2d *distortedCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool distorted2undistortedImage(TUVision::CVertex2d &distortedCoords, TUVision::CVertex2d *undistortedCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool world2image(TUVision::CVertex3d &worldCoords, TUVision::CVertex2d *imageCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool image2world(TUVision::CVertex2d &imageCoords, double worldZValue, TUVision::CVertex3d *worldCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool world2camera(TUVision::CVertex3d &worldCoords, TUVision::CVertex3d *cameraCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static bool camera2world(TUVision::CVertex3d &cameraCoords, TUVision::CVertex3d *worldCoords, SCameraParameters &cameraParameters, SCalibrationConstants &calibrationConstants);
    static void euler2Matrix(SCalibrationConstants *calibrationConstants);
private:
    SCameraParameters *d_cameraParameters;
    SCalibrationConstants *d_calibrationConstants;
};

}

#endif // __TUVISION_TSAITRANSFORM_H__
