/***************************************************************************
                          CColourSpaceMatrix.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv/CColourSpaceMatrix.h"
#include <cassert>

using namespace TUVision;

/*! 
 *  specific to colourspacematrices,
 *  \param inchannels specifies the number of columns
 *  \param outchannels specifies the number of rows
 */
CColourSpaceMatrix::CColourSpaceMatrix(int inchannels, int outchannels)
: d_inchannels(inchannels),
d_outchannels(outchannels)
{
    assert(inchannels>0);
    assert(outchannels>0);

    d_data=new double[d_inchannels*outchannels];
}


CColourSpaceMatrix::~CColourSpaceMatrix()
{
    delete [] d_data;
}

/** multiplies right argument into this matrix.
 *  \bug does not do anything yet except trashing this matrix' contents.
 */
CColourSpaceMatrix& 
CColourSpaceMatrix::operator*=(const CColourSpaceMatrix& cm)
{
    assert(d_inchannels==cm.d_outchannels);

    double* new_data=new double[d_outchannels*cm.d_inchannels];

    /// CALCULATE

    delete [] d_data;

    /// adjust sizes
    d_data=new_data;
    d_inchannels=cm.d_inchannels;


    return *this;
}


/*! returns the row vector to convert into one destination channel.
 * \param channel defines the outputchannel (0..outchannels-1)
 * \return double vector of length inchannels.
 */
const double*
CColourSpaceMatrix::getChannelRow(int channel) const
{
    assert(channel<d_outchannels);
    assert(channel>=0);


    return d_data+d_inchannels*channel;
}
