/***************************************************************************
                          CMinpack.cpp  -  description
                             -------------------
    begin                : 06.08.2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : PhD Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include "minpack/minpack.h"
#undef abs
#undef min
#undef max

#include "tuv/CMinpack.h"

using namespace TUVision;

static CMinpack *p_currentinstance;


static int callback(integer *m_ptr, integer *n_ptr, doublereal *x, doublereal *fvec, integer *iflag)
{
    int i;
    for(i=0;i<*n_ptr;++i)
        p_currentinstance->m_solution[i] = x[i];

    (p_currentinstance->m_costfunction)(p_currentinstance->m_solution,&p_currentinstance->m_costvector,
        p_currentinstance->m_userdata);

    for(i=0;i<*m_ptr;++i)
        fvec[i] = p_currentinstance->m_costvector[i];

    return 0;
}

CMinpack::CMinpack(Tcostfunction costfunction, std::vector<double> initialguess, int costvectorlength) :
m_costfunction(costfunction),
m_costvector(costvectorlength),
m_solution(initialguess),
m_iterations(0),
m_maxiterations(10000)
{
}

std::vector<double> &CMinpack::run(void *userdata)
{
    m_userdata     = userdata;
    integer        m = m_costvector.size();
    integer        n = m_solution.size(); // Fundamental matrix has 9 entries
    doublereal *   x = new doublereal[n];  // Fundamental matrix has 9 entries
    doublereal *fvec = new doublereal[m];
    doublereal  ftol = 1.0E-9; // Termination criterion: If the global error is less than ftol, then stop.
    doublereal  xtol = 1.0E-7; // Termination criterion: If an additional step gives less than xtol, then stop.
    doublereal  gtol = 0.0;    // Some angle between the jacobian and something else: We don't want this.
    integer     maxfev = m_maxiterations;// Maximum iteration
    doublereal  epsfcn = 1.0E-16;// Minimum stepsize for x (not zero!)
    doublereal   *diag = new doublereal[m];       // see lmdif.f
    integer     mode = 1;      // Variables are scaled internally
    doublereal  factor = 100.0;
    integer     nprint = 0;
    integer info;
    integer nfev;
    doublereal *fjac = new doublereal[m*n];
    integer ldfjac = m;
    integer    *ipvt = new integer[n];
    doublereal  *qtf = new doublereal[n];
    doublereal  *wa1 = new doublereal[n];
    doublereal  *wa2 = new doublereal[n];
    doublereal  *wa3 = new doublereal[n];
    doublereal  *wa4 = new doublereal[m];

    p_currentinstance = this;

    for(unsigned int i=0;i<m_solution.size();++i)
        x[i] = m_solution[i];

    lmdif_ (callback,&m,&n,x,fvec,&ftol,&xtol, &gtol, &maxfev, &epsfcn,
        diag, &mode, &factor, &nprint, &info, &nfev, fjac, &ldfjac,
        ipvt, qtf, wa1, wa2, wa3, wa4);

    m_iterations = nfev;

    delete [] fvec;
    delete [] fjac;

    delete [] ipvt;
    delete [] qtf;
    delete [] wa1;
    delete [] wa2;
    delete [] wa3;
    delete [] wa4;

    delete [] x;
    delete [] diag;

    return m_solution;
}

void CMinpack::setMaxIterations  (int maxiterations)
{
    m_maxiterations = maxiterations;
}

int  CMinpack::getNumOfIterations()
{
    return m_iterations;
}
