/***************************************************************************
                          CPoint.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include "tuv/CPoint.h"

using namespace TUVision;


/**********************************************************/
/*!
 *  The default constructor sets the coordinates of the
 *  created point to (0,0).
 */
/**********************************************************/

CPoint::CPoint () 
: d_x (0),
d_y (0)
{
}


/**********************************************************/
/*!
 *  This constructor sets the coordinates of the created
 *  point to (x,y).
 *
 *  \param x x-coordinate
 *  \param y y-coordinate
 */
/**********************************************************/

CPoint::CPoint (int x, int y) 
: d_x (x),
d_y (y)
{
}


/**********************************************************/
/*!
 *  The copy constructor creates a new point with the
 *  coordinates of an existing CPoint object.
 *
 *  \param pnt CPoint object to be copied
 */
/**********************************************************/

CPoint::CPoint (const CPoint &pnt)
: d_x (pnt.d_x),
d_y (pnt.d_y)
{
}


/**********************************************************/
/*!
 *   Sets the coordinates of the object to (x,y).
 *
 *   \param x x-coordinate
 *   \param y y-coordinate
 */
/**********************************************************/

void CPoint::setPoint (int x, int y)
{
    d_x = x;
    d_y = y;
}


/**********************************************************/
/*!
 *   Sets the x-coordinate of the point.
 *
 *   \param x x-coordinate
 */
/**********************************************************/

void CPoint::setX (int x)
{
    d_x = x;
}


/**********************************************************/
/*!
 *   Sets the y-coordinate of the point.
 *
 *   \param x y-coordinate
 */
/**********************************************************/

void CPoint::setY (int y)
{
    d_y = y;
}


/**********************************************************/
/*!
 *   Copies the coordinates of an existing CPoint object
 *   to this object.
 *
 *   \param rhs coordinates to be copied
 */
/**********************************************************/

CPoint &CPoint::operator= (const CPoint &rhs)
{
    if (this == &rhs)
        return *this;

    d_x = rhs.d_x;
    d_y = rhs.d_y;

    return *this;
}


/**********************************************************/
/*!
 *   Tests whether both points have got the same
 *   x- and y-coordinates.
 *
 *   \param pt point used for the comparision
 */
/**********************************************************/

bool CPoint::operator== (const CPoint &pt) const
{
    if (d_x != pt.getX())
        return (false);

    if (d_y != pt.getY())
        return (false);

    return (true);
}


/**********************************************************/
/*!
 *   Tests whether both points have got not the same
 *   x- and y-coordinates.
 *
 *   \param pt point used for the comparision
 */
/**********************************************************/

bool CPoint::operator!= (const CPoint &pt) const
{
    if (d_x != pt.getX())
        return (true);

    if (d_y != pt.getY())
        return (true);

    return (false);
}

/**********************************************************/
/*!
 *   subtracts the operand from this point.
 *   
 *   \param p point used for the subtraction
 */
/**********************************************************/
CPoint& 
CPoint::operator-=(const CPoint& p)
{
    d_x-=p.d_x;
    d_y-=p.d_y;
    return *this;
}

/**********************************************************/
/*!
 *   adds the operand to this point.
 *   
 *   \param p point used for the addition
 */
/**********************************************************/
CPoint& 
CPoint::operator+=(const CPoint& p)
{
    d_x+=p.d_x;
    d_y+=p.d_y;
    return *this;
}

/**********************************************************/
/*!
 *   Tests whether both points have got not the same
 *   x- and y-coordinates.
 *
 *   \param pt point used for the comparision
 */
/**********************************************************/
int 
CPoint::getLength() const
{
    return d_x*d_x+d_y*d_y;
}

