/***************************************************************************
                          CPointShape.cpp  -  description
                             -------------------
    begin                : 10.04.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library's internal version. It is   *
 *   not permitted to publish this file at this point of time as the       *
 *   algorithms are meant to be shared between TUHH Vision Systems         *
 *   members only.                                                         *
 *                                                                         *
 ***************************************************************************/

#include "tuv/CPointShape.h"

using namespace TUVision;

/**********************************************************/
/*!
 *  The default constructor sets the bounding box of the
 *  shape created point to (0,0). The list of shape points
 *  is created as empty
 */
/**********************************************************/
CPointShape::CPointShape() : d_BoundingBox(0,0){
    d_ShapePoints = new list<CPoint>;
    d_empty=true;
}


/**********************************************************/
/*!
 *  Copy Constructor
 */
/**********************************************************/
CPointShape::CPointShape(const CPointShape &other) :
d_empty(other.d_empty),
d_BoundingBox(other.d_BoundingBox),
d_ShapePoints(new list<CPoint>(*other.d_ShapePoints))
{
}


/**********************************************************/
/*!
 *  The destructor deletes all points in the point list as
 *  they won't be needed anymore
 */
/**********************************************************/
CPointShape::~CPointShape() {
    delete d_ShapePoints;
}


/**********************************************************/
/*!
 *  Adds a new point to the list of shape points
 *  and updates the bounding box accordingly so that
 *  all point (including this new one) are located inside
 *  the bouding box
 *  \param newPoint the new point which is added to the shape
 */
/**********************************************************/
void CPointShape::addPointUpdateBBX(CPoint &newPoint){
    if(d_empty){
        d_BoundingBox.setRect(newPoint,newPoint);
        d_empty=false;
    }
    else{

        // Adjust new bounding box values
        if(d_BoundingBox.getX() > newPoint.getX())
            d_BoundingBox.setRect(newPoint.getX(),d_BoundingBox.getY(),d_BoundingBox.getLRX(),d_BoundingBox.getLRY());

        else if(d_BoundingBox.getLRX() < newPoint.getX())
            d_BoundingBox.setRect(d_BoundingBox.getX(),d_BoundingBox.getY(),newPoint.getX(),d_BoundingBox.getLRY());

        if(d_BoundingBox.getY() > newPoint.getY())
            d_BoundingBox.setRect(d_BoundingBox.getX(),newPoint.getY(),d_BoundingBox.getLRX(),d_BoundingBox.getLRY());

        else if(d_BoundingBox.getLRY() < newPoint.getY())
            d_BoundingBox.setRect(d_BoundingBox.getX(),d_BoundingBox.getY(),d_BoundingBox.getLRX(),newPoint.getY());
    }

    d_ShapePoints->push_back(newPoint);
}

/**********************************************************/
/*!
 *  Tests whether the shape contains a point. This method does
 *  not produce absolutely correct values as it is only checked
 *  whether the point lies inside the shape's bounding box.
 *
 *  \param testpoint the point which is used for the test
 */
/**********************************************************/
bool CPointShape::containsPoint(CPoint &testpoint){
    return d_BoundingBox.contains(testpoint);
}
