/***************************************************************************
                          CFileReadPng.cpp  -  description
                             -------------------
    begin                : 19.09.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Preparation Bildverarbeitung 1 WS 2001

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include <string>
#include <cstdio>
#include <exception>

#include <png.h>

#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CFileReadPng.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"

using namespace std;
using namespace TUVision;

const unsigned char CFileReadPngRegistrator::d_id[]={(unsigned char) 0x89,'P','N','G'};
const int CFileReadPngRegistrator::d_len=4;

CFileReadPng::CFileReadPng (void)
{
    d_filename = "";
    d_hd = NULL;
}


CFileReadPng::~CFileReadPng()
{
    if (d_hd != NULL)
        fclose (d_hd);
}


void CFileReadPng::open (const string &filename)
throw (CException)
{
    d_filename = filename;

    d_hd = fopen (d_filename.c_str(), "rb");
    if (d_hd == NULL)
        throw CExceptionFileOpen (d_filename);
}


void CFileReadPng::close (void)
throw()
{
    if (d_hd != NULL)
    {
        fclose (d_hd);
        d_hd = NULL;
    }
}


CImage *CFileReadPng::read (int img_number)
throw (CException, exception)
{
    png_uint_32 width, height;
    png_uint_32 y;
    int bit_depth, color_type;
    int interlace_type, compression_type, filter_type;

    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);
    png_infop info_ptr = png_create_info_struct(png_ptr);
    png_infop end_info = png_create_info_struct(png_ptr);

    CImageByte                     *img = NULL;
    png_bytep *row_pointers = NULL;


    // Valid file handle?
    if (d_hd == NULL)
        throw CExceptionFileRead (d_filename);

    // Install our own error handler. This handler is used
    // to get a description of the problem and throw an
    // exception. - has to be done?!

    if(setjmp(png_jmpbuf(png_ptr))){
        png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);
        if (img != NULL)
            delete img;
        throw CExceptionFileRead( d_filename );
    }

    png_init_io(png_ptr,d_hd);
    png_read_info(png_ptr,info_ptr);
    png_get_IHDR(png_ptr,info_ptr,&width,&height,&bit_depth,&color_type,&interlace_type,&compression_type,&filter_type);

    if(color_type == PNG_COLOR_TYPE_PALETTE)
        png_set_palette_to_rgb(png_ptr);
    if(color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
        png_set_gray_1_2_4_to_8(png_ptr);
    if(bit_depth == 16)
        png_set_strip_16(png_ptr);
    png_read_update_info(png_ptr,info_ptr);

    // The PNG - docs state that it would be better to combine the alpha
    // - channel with the background, but there's no indication of how telling
    // libpng to do so. So we just discard it.
    if(color_type & PNG_COLOR_MASK_ALPHA)
        png_set_strip_alpha(png_ptr);

    switch (color_type) {
  case PNG_COLOR_TYPE_GRAY:
      // Read a 8 bit gray image.
      { 
          try {
              img = new CImageByteGray (width, height);
          }
          catch (...) {
              png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);
              throw;
          }
      }
      break;
  case PNG_COLOR_TYPE_PALETTE:
  case PNG_COLOR_TYPE_RGB_ALPHA:
  case PNG_COLOR_TYPE_RGB:
      // Read a 8 bit RGB image.
      { 
          try {
              img = new CImageByteRGB (width, height);
          }
          catch (...) {
              png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);
              throw;
          }
      }
      break;
  default:
      // unsupported image format
      {
          png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);
          throw CExceptionFileRead (d_filename, 
              "The color model or the resolution of the image "
              "is not supported.");
      }
    }

    row_pointers = new png_bytep[height];

    for(y=0;y<height;y++)
        row_pointers[y] = img->getRow(y);

    png_read_image(png_ptr,row_pointers);

    png_read_end(png_ptr,end_info);
    png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);

    delete [] row_pointers;
    return (img);
} 


int CFileReadPng::getNumberImages (void)
throw()
{
    return (1);
}


