/***************************************************************************
                          CEllipse.cpp  -  description
                             -------------------
    begin                : 21.9.2002
    copyright            : (C) 2002 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CEllipse.h"
using namespace TUVision;

/**********************************************************/
/*!
 *  The creates ellipses with distance focal points varies
 *  from c0 to c1 and the ratio between b to a is BA = b/a
 *  0 < BA < 1
 *	y^2/a^2 + x^2/b^2 = 1
 * 	b^2 = a^2 - c^2
 *  c = a - b = a - (BA*a)
 *  a = c/sqrt(1-BA^2)
 *  
 *  \param a0 the minimum distance between fuci
 *  \param a1 the maximum distance between fuci
 *  \param AB ratio of a to b of the ellips
 */
/**********************************************************/

void
CEllipse::setEllipses(int a0, int a1, double BA, double direction,
                      int Q1, int Q2,int Q3,int Q4,int OriginPosition, int step)
{

    // y^2/a^2 + x^2/b^2 = 1
    // b^2 = a^2 - c^2
    //

    double ba = 1/sqrt(1-BA*BA);
    for(double a = a0; a <= a1; a +=step)
    {
        d_Cxy.vecCurvePoints.clear(); // empty points
        double b = a*BA; // b
        double c = sqrt(a*a - b*b);//c*ba; // c
        int CurveLength=0;
        int x1 = (int) b;
        double dx = 0;
        double dy = a;
        //double dy = a;//-1.2*c;//a;
        if(direction > 0) dy = b;
        if(direction == 0) // in y axis
        {
            if(OriginPosition == 0) // top
            {
                dx = 0;
                dy = a;
            }
            if(OriginPosition == 1) // left
            {
                dx = b;
                dy = 0;
            }
            if(OriginPosition == 2) // bottom
            {
                dx = 0;
                dy = -a;
            }
            if(OriginPosition == 3) // right
            {
                dx = -b;
                dy = 0;
            }
        }
        if(direction > 0) // in x axis
        {
            if(OriginPosition == 0) // top
            {
                dx = 0;
                dy = b;
            }
            if(OriginPosition == 1) // left
            {
                dx = a;
                dy = 0;
            }
            if(OriginPosition == 2) // bottom
            {
                dx = 0;
                dy = -b;
            }
            if(OriginPosition == 3) // right
            {
                dx = -a;
                dy = 0;
            }
        }
        for(int y = 0; y <= a; y++)
        {
            int x = (int)(b*sqrt(1 - (double)(y*y)/(a*a) ) );
            // check the derivative 2y/a^2 --> 2x/b^2
            if(2*y/(a*a) > 2*x/(b*b))
            {
                x1 = x;
                break;
            }
            int X = x , Y = y;
            if(direction > 0) // a is in x
            {
                X = y; Y = x;
            }
            // Q1
            if(Q1 >0)
            {
                d_Pxy.x = (int) (X + dx); //store this point in the curve
                d_Pxy.y = (int) (Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
            // Q2
            if(Q2 >0)
            {
                d_Pxy.x = (int) (-X + dx); //store this point in the curve
                d_Pxy.y = (int) (Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
            // Q3
            if(Q3 >0)
            {
                d_Pxy.x = (int) (X + dx); //store this point in the curve
                d_Pxy.y = (int) (-Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
            // Q4
            if(Q4 >0)
            {
                d_Pxy.x = (int) (-X + dx); //store this point in the curve
                d_Pxy.y = (int) (-Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
        }
        for(int x = x1; x >= 0; x--)
        {
            int y = (int)(a*sqrt(1 - (double)(x*x)/(b*b)) );
            // check the derivative 2y/a^2 --> 2x/b^2
            //if(2*y/(a*a) > 2*x/(b*b)) break;
            int X = x , Y = y;
            if(direction > 0) // a is in x
            {
                X = y; Y = x;
            }
            // Q1
            if(Q1 >0)
            {
                d_Pxy.x = (int) (X + dx); //store this point in the curve
                d_Pxy.y = (int) (Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
            // Q2
            if(Q2 >0)
            {
                d_Pxy.x = (int) (-X + dx); //store this point in the curve
                d_Pxy.y = (int) (Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }

            // Q3
            if(Q3 >0)
            {
                d_Pxy.x = (int) (X + dx); //store this point in the curve
                d_Pxy.y = (int) (-Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
            // Q4
            if(Q4 >0)
            {
                d_Pxy.x = (int) (-X + dx); //store this point in the curve
                d_Pxy.y = (int) (-Y + dy);
                d_Cxy.vecCurvePoints.push_back(d_Pxy); // add a point to the curve
                CurveLength++; // increase the length of the curve
            }
        }

        d_Cxy.CurveLength=CurveLength; // set the curve length
        d_Cxy.CurveEndPoint = d_Pxy;   // set the curve end point
        d_Pxy.x = 0; d_Pxy.x = 0;
        d_Cxy.CurveOrigin   = d_Pxy;   // origin
        d_Cxy.attribute = a;           // the distance between focis
        d_Cxy.parameters[0] = a;
        d_Cxy.parameters[1] = b;
        d_Cxy.parameters[2] = c;
        d_Cxy.parameters[3] = direction; // 0 is in y direction, 1 in x direction
        d_Cxy.parameters[4] = BA; // BA b/a ratio
        vecCurvesLUT.push_back(d_Cxy); // add the curve to LUT
        d_Cxy.vecCurvePoints.clear();
    }// end for X	
    getRidOfDuplicatedCurves();	// this curve is stored here in the LUT
}

/// construct a Parabola object with heignt h and width w
CEllipse::CEllipse(int w,int h,int x_direction , double thr0 , double thr1)
{

}
