/***************************************************************************
                          CCannyFilter.cpp  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include <memory> 
#include "tuv2d/CCannyFilter.h"
#include "tuv/CImageByteGray.h"
#include "libcanny.h"


extern "C" {

    //extern	float ratio = 0.1;
    //extern	int WIDTH = 0;

    //extern	int trace (int i, int j, int low, IMAGE im,IMAGE mag, IMAGE ori);
    //extern	float gauss(float x, float sigma);
    //extern	float dGauss (float x, float sigma);
    //extern	float meanGauss (float x, float sigma);
    extern	void hysteresis (int high, int low, IMAGE im, IMAGE mag, IMAGE oriim);
    extern	void canny (float s, IMAGE im, IMAGE mag, IMAGE ori);
    //extern	void seperable_convolution (IMAGE im, float *gau, int width, 
    //			float **smx, float **smy);
    //extern	void dxy_seperable_convolution (float** im, int nr, int nc, float *gau,  
    //			int width, float **sm, int which);
    //extern	void nonmax_suppress (float **dx, float **dy, int nr, int nc, 
    //			IMAGE mag, IMAGE ori);
    //extern	void estimate_thresh (IMAGE mag, int *low, int *hi);
    extern	int getWIDTH();
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
using namespace TUVision;

CCannyFilter::CCannyFilter(float s_in, int low_in, int high_in)
{
    s= s_in; //! standard deviation
    low= low_in; //! low edge and high edge
    high= high_in; //! low edge and high edge
    im = 0;
    magim = 0;
    oriim = 0;
}

void
CCannyFilter::init(CImageByteGray &img,float s_in, int low_in, int high_in)
{
    s= s_in; //! standard deviation
    low= low_in; //! low edge and high edge
    high= high_in; //! low edge and high edge

    if(im != 0)freeCannyImage(im);
    if(magim != 0) freeCannyImage(magim);
    if(oriim != 0)freeCannyImage(oriim);

    im = newCannyImage(img);
    magim = newCannyImage(img, false); //(im->info->nr, im->info->nc);
    oriim = newCannyImage(img, false); //(im->info->nr, im->info->nc);
}

CCannyFilter::~CCannyFilter()
{
    freeCannyImage(im);
    freeCannyImage(magim);
    freeCannyImage(oriim);
}

void
CCannyFilter::filter (CImageByteGray &img)
{
    int i,j,k,n;
    //	float s=1.0;
    //	int low= 0,high=10;
    //	FILE *params;

    /* Command line: input file name */
    /*	if (argc < 2)
    {
    printf ("USAGE: canny <filename>\n");
    printf ("Canny edge detector - reads a PGM format file and\n");
    printf (" detects edges, creating 'canny.pgm'.\n");
    exit (1);
    }
    printf ("CANNY: Apply the Canny edge detector to an image.\n");
    */
    /* Read parameters from the file canny.par */

    /*
    params = fopen ("canny.par", "r");
    if (params)
    {
    fscanf (params, "%d", &low);  // Lower threshold 
    fscanf (params, "%d", &high); // High threshold 
    fscanf (params, "%f", &s);   // Gaussian standard deviation 
    printf ("Parameters from canny.par: HIGH: %d LOW %d Sigma %f\n",
    high, low, s);
    fclose (params);
    }
    else printf ("Parameter file 'canny.par' does not exist.\n");
    */

    /* Read the input file */
    //	im = newCannyImage(img);
    /*
    if (im == 0)
    {
    printf ("No input image ('%s')\n", argv[1]);
    exit (2);
    }
    */
    /* Create local image space */
    //	magim = newCannyImage(img, false); //(im->info->nr, im->info->nc);
    /*
    if (magim == NULL)
    {
    printf ("Out of storage: Magnitude\n");
    exit (1);
    }
    */
    //	oriim = newCannyImage(img, false); //(im->info->nr, im->info->nc);
    /*
    if (oriim == NULL)
    {
    printf ("Out of storage: Orientation\n");
    exit (1);
    }
    */

    /* Apply the filter */
    copyImageToCanny(img,im);
    canny (s, im, magim, oriim);

    /* Hysteresis thresholding of edge pixels */
    hysteresis (high, low, im, magim, oriim);
    for (i=0; i<getWIDTH(); i++)
        for (j=0; j<im->info->nc; j++)
            im->data[i][j] = 255;

    for (i=im->info->nr-1; i>im->info->nr-1-getWIDTH(); i--)
        for (j=0; j<im->info->nc; j++)
            im->data[i][j] = 255;

    for (i=0; i<im->info->nr; i++)
        for (j=0; j<getWIDTH(); j++)
            im->data[i][j] = 255;

    for (i=0; i<im->info->nr; i++)
        for (j=im->info->nc-getWIDTH()-1; j<im->info->nc; j++)
            im->data[i][j] = 255;
    /*
     */
    //	Output_PBM (im, "canny.pgm");
    copyCannyImageTo(img, im);

    //	printf ("Output file is:\n");
    //	printf ("  canny.pgm - edge-only image\n");

}

IMAGE
CCannyFilter::newCannyImage(CImageByteGray &img,bool cpy)
{
    struct image  *x;                /* New image */
    unsigned char *ptr;             /* new pixel array */
    int i;
    int nr = img.getH();
    int nc = img.getW();

    if (nr < 0 || nc < 0) {
        printf ("Error: Bad image size (%d,%d)\n", nr, nc);
        return 0;
    }

    /*      Allocate the image structure    */
    x = (struct image  *) malloc( sizeof (struct image) );
    if (!x) {
        printf ("Out of storage in NEWIMAGE.\n");
        return 0;
    }

    /*      Allocate and initialize the header      */

    x->info = (struct header *)malloc( sizeof(struct header) );
    if (!(x->info)) {
        printf ("Out of storage in NEWIMAGE.\n");
        return 0;
    }
    x->info->nr = nr;       x->info->nc = nc;
    x->info->oi = x->info->oj = 0;

    /*      Allocate the pixel array        */

    x->data = (unsigned char **)malloc(sizeof(unsigned char *)*nr); 

    /* Pointers to rows */
    if (!(x->data)) {
        printf ("Out of storage in NEWIMAGE.\n");
        return 0;
    }

    for (i=0; i<nr; i++) 
    {
        x->data[i] = (unsigned char *)malloc(nc*sizeof(unsigned char));
        if (cpy)
            memcpy(x->data[i],img.getRow(i), nc);
        /*
        if (x->data[i]==0)
        {
        printf ("Out of storage. Newimage - row %d\n", i);
        exit(1);
        }
        */
    }
    return x;
}


void
CCannyFilter::copyCannyImageTo(CImageByteGray& img, IMAGE x)
{
    int h = img.getH();
    int w = img.getW();
    for (int i=0; i<h; i++) 
    {
        memcpy(img.getRow(i), x->data[i], w);
    }

};

void
CCannyFilter::copyImageToCanny(CImageByteGray& img, IMAGE x)
{
    int h = x->info->nr;
    int w = x->info->nc;
    for (int i=0; i<h; i++) 
    {
        memcpy(x->data[i], img.getRow(i), w);
    }

};


void
CCannyFilter::freeCannyImage (struct image  *z)
{
    /*      Free the storage associated with the image Z    */
    int i;

    if (z != 0) 
    {
        for (i=0; i<z->info->nr; i++)
            free (z->data[i]);
        free (z->info);
        free (z->data);
        free (z);
    }

}
