/***************************************************************************
                           CFilterMaxMin.cpp  -  description
                              -------------------
     begin                : 30.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <tuv2d/CFilterMaxMin.h>

// using namespace std;
using namespace TUVision;


/**************************************************
 *
 * Constructor and destructor
 *
 **************************************************/

//
//! Constructor (sets filter width to 3 if called without argument).
//

CFilterMaxMin::CFilterMaxMin(unsigned char width)
{
    filterWidth = width;
}


//
//! Destructor.
//

CFilterMaxMin::~CFilterMaxMin()
{
}


/**************************************************
 *
 * Public methods:
 *
 * - get and set filter width
 * - filter an image
 *
 **************************************************/

//
//! Set filter width (must be odd value).
//

bool CFilterMaxMin::setWidth(unsigned char width)
{
    if (width%2!=1) return false;			// Input value is even
    else			filterWidth = width;	// Input value is odd
    return true;
}


//
//! Returns filter width.
//

unsigned char CFilterMaxMin::getWidth()
{
    return filterWidth;
}


/*
 * Filter image
 */

/*
 *	Meaning of 'offset' (e.g. filterWidth = 7):
 *
 *	offset=3
 *	+ - - - - - + <- middle - offset
 *	|     |     |
 *	|     |     |
 *	| - - P - - | <- middle
 *	|     |     |
 *	|     |     |
 *	+ - - - - - + <- middle + offset
 *
 *	P = pixel to be filtered (filter window centered)
 */

//
//! Filter the image (maximum).
//

void CFilterMaxMin::filterMax(CImageByteGray &img)
{

    if(!(filterWidth%2)) return;			// Filter length has to be odd

    unsigned char	offset = filterWidth/2;	//!< See drawing in source-code
    int				x_win,y_win;			//!< Relative position in filter window
    unsigned char	*row;					//!< Pointer: 1st pixel in row (image matrix)
    unsigned char	max;					//!< Maximum value in filter window
    CImageByteGray	*orig_img;				//!< Original image

    orig_img = new CImageByteGray (img);
    img.clearImage();

    // Run through pixels (positions to be filtered)
    for (int y=offset; y<orig_img->getH()-offset; y++)
    {
        for (int x=offset; x<orig_img->getW()-offset; x++)
        {
            // Evaluate filter window
            max = orig_img->getPixel(-offset,y);
            for (y_win=-offset; y_win<=offset; y_win++)
            {
                row = orig_img->getRow(y+y_win);
                for (x_win=-offset; x_win<=offset; x_win++)
                    if (row[x+x_win]>max) max = row[x+x_win];
            }
            img.setPixel(x,y,max,0);
        }
    }
    delete orig_img;
}


//
//! Filter the image (minimum).
//

void CFilterMaxMin::filterMin(CImageByteGray &img)
{

    if(!(filterWidth%2)) return;			// Filter length has to be odd

    unsigned char	offset = filterWidth/2;	//!< See drawing in source-code
    int				x_win,y_win;			//!< Relative position in filter window
    unsigned char	*row;					//!< Pointer: 1st pixel in row (image matrix)
    unsigned char	min;					//!< Minimum value in filter window
    CImageByteGray	*orig_img;				//!< Original image

    orig_img = new CImageByteGray (img);
    img.clearImage();

    // Run through pixels (positions to be filtered)
    for (int y=offset; y<orig_img->getH()-offset; y++)
    {
        for (int x=offset; x<orig_img->getW()-offset; x++)
        {
            // Evaluate filter window
            min = orig_img->getPixel(x,-offset);
            for (y_win=-offset; y_win<=offset; y_win++)
            {
                row = orig_img->getRow(y+y_win);
                for (x_win=-offset; x_win<=offset; x_win++)
                    if (row[x+x_win]<min) min = row[x+x_win];
            }
            img.setPixel(x,y,min,0);
        }
    }
    delete orig_img;
}

