/***************************************************************************
                           COperatorFlipImage.cpp  -  description
                              -------------------
     begin                : 22.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Study thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <tuv2d/COperatorFlipImage.h>
#include <tuv/CImageByteGray.h>

// using namespace std;
using namespace TUVision;


/**************************************************
 *
 * Constructor and destructor
 *
 **************************************************/

COperatorFlipImage::COperatorFlipImage()
{
}


COperatorFlipImage::~COperatorFlipImage()
{
}


/**************************************************
 *
 * Public methods
 *
 **************************************************/

/*
 * Flip gray image
 */

void COperatorFlipImage::flip(CImageByteGray *img,bool horizontal,bool vertical)
{
    if ((!horizontal)&&(!vertical)) return;

    unsigned char	*row,*orig_row;		// Pointer: 1st pixel in row (image matrix)
    int				orig_x,orig_y;
    int				x,y;
    int				x_max,y_max;		// Constants (Avoid calculation in each for-loop)

    CImageByteGray *orig_img = new CImageByteGray(*img);

    if ((horizontal)&&(!vertical))		// Flip horizontal
    {
        x_max = img->getW() - 1;
        y_max = img->getH();
        for (y=0; y<y_max; y++)
        {
            orig_row	= orig_img->getRow(y);
            row			= img->getRow(y);
            for (orig_x=0,x=x_max; x>=0; orig_x++,x--)
                row[x] = orig_row[orig_x];
        }
    }

    else if ((!horizontal)&&(vertical))	// Flip vertical
    {
        x_max = img->getW();
        y_max = img->getH() - 1;
        for (y=y_max,orig_y=0; y>=0; y--,orig_y++)
        {
            orig_row	= orig_img->getRow(orig_y);
            row			= img->getRow(y);
            for (x = 0; x<x_max; x++)
                row[x] = orig_row[x];
        }
    }

    else if ((horizontal)&&(vertical))	// Flip horizontal and vertical
    {
        x_max = img->getW() - 1;
        y_max = img->getH() - 1;
        for (y=y_max,orig_y=0; y>=0; y--,orig_y++)
        {
            orig_row	= orig_img->getRow(orig_y);
            row			= img->getRow(y);
            for (orig_x=0,x=x_max; x>=0; orig_x++,x--)
                row[x] = orig_row[orig_x];
        }
    }

    delete(orig_img);
}


/*
 * Flip RGB image
 */

void COperatorFlipImage::flip(CImageByteRGB *img,bool horizontal,bool vertical)
{
    if ((!horizontal)&&(!vertical)) return;

    unsigned char	*row,*orig_row;		// Pointer: 1st pixel in row (image matrix)
    int				orig_x,orig_y;
    int				x,y;
    int				x_max,y_max;		// Constants (Avoid calculation in each for-loop)

    CImageByteRGB *orig_img = new CImageByteRGB(*img);

    if ((horizontal)&&(!vertical))		// Flip horizontal
    {
        x_max = 3*img->getW() - 3;
        y_max = img->getH();
        for (y=0; y<y_max; y++)
        {
            orig_row	= orig_img->getRow(y);
            row			= img->getRow(y);
            for (orig_x=0,x=x_max; x>=0; orig_x+=3,x-=3)
            {
                row[x]   = orig_row[orig_x];
                row[x+1] = orig_row[orig_x+1];
                row[x+2] = orig_row[orig_x+2];
            }
        }
    }

    else if ((!horizontal)&&(vertical))	// Flip vertical
    {
        x_max = 3*img->getW();
        y_max = img->getH() - 1;
        for (y=y_max,orig_y=0; y>=0; y--,orig_y++)
        {
            orig_row	= orig_img->getRow(orig_y);
            row			= img->getRow(y);
            for (x=0; x<x_max; x+=3)
            {
                row[x]   = orig_row[x];
                row[x+1] = orig_row[x+1];
                row[x+2] = orig_row[x+2];
            }
        }
    }

    else if ((horizontal)&&(vertical))	// Flip horizontal and vertical
    {
        x_max = 3*img->getW() - 3;
        y_max = img->getH() - 1;
        for (y=y_max,orig_y=0; y>=0; y--,orig_y++)
        {
            orig_row	= orig_img->getRow(orig_y);
            row			= img->getRow(y);
            for (orig_x=0,x=x_max; x>=0; orig_x+=3,x-=3)
            {
                row[x]   = orig_row[orig_x];
                row[x+1] = orig_row[orig_x+1];
                row[x+2] = orig_row[orig_x+2];
            }
        }
    }

    delete(orig_img);
}
